/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XHTML + Voice support.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Cross <xcross@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#ifndef nsVxmlFormElement_h
#define nsVxmlFormElement_h

#include "nsIDOMEventListener.h"
#include "nsIXTFGenericElement.h"
#include "nsIVxmlFormElement.h"
#include "nsIVxmlInterpreter.h"

class nsVxmlFormElement : public nsIVxmlFormElement, 
                          public nsIXTFGenericElement,
                          public nsIDOMEventListener
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIVXMLFORMELEMENT
  NS_DECL_NSIXTFELEMENT
  NS_DECL_NSIXTFGENERICELEMENT
  NS_DECL_NSIDOMEVENTLISTENER

  nsVxmlFormElement();

private: // functions
  ~nsVxmlFormElement();
  nsresult Activate(class nsIDOMEvent *aEvent);
  nsresult OnDOMContentLoaded();
  nsresult Serialize(nsAString& aString);	
  long getTimeStamp() {return mCount++;}
  void init();

private: // variables
  nsIDOMElement * mElement;
  long mCount;
  nsCOMPtr<nsIDocument> mDoc;
  nsString mID;
  nsString mDocURI;
  nsString mBaseURI;
  nsIVxmlInterpreter * mVxmlInterpreter;
  PRBool mbDoneCreatingDOM;

  /** Blindly copied from nsXFormsUtils
   * Clone the set of IIDs in |aIIDList| into |aOutArray|.
   * |aOutCount| is set to |aIIDCount|.
   */
  static NS_HIDDEN_(nsresult) CloneScriptingInterfaces(const nsIID *aIIDList,
                                                       unsigned int aIIDCount,
                                                       PRUint32 *aOutCount,
                                                       nsIID ***aOutArray);
protected:
  /* additional members */
};


NS_HIDDEN_(nsresult) NS_NewVxmlFormElement(nsIXTFElement **aResult);

#endif //#define nsVxmlFormElement_h



