/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XHTML + Voice support.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Cross <xcross@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <stdio.h>
#include "jsapi.h"
#include "nsString.h"
#include "nsCOMPtr.h"
#include "nsIDocument.h"
#include "nsIDOMDocument.h"
#include "nsIDOMElement.h"
#include "nsIDOMEvent.h"
#include "nsIDOMEvent.h"
#include "nsIDOM3Node.h"
#include "nsIDOMNodeList.h"
#include "nsIDOMSerializer.h"
#include "nsIJSContextStack.h"
#include "nsIPrincipal.h"
#include "nsIScriptContext.h"
#include "nsIScriptGlobalObject.h"
#include "nsIScriptObjectPrincipal.h"
#include "nsIServiceManager.h"
#include "nsIXTFElement.h"
#include "nsXvHandlerElement.h"
#include "nsIXTFElement.h"
#include "nsIXTFGenericElementWrapper.h"

NS_IMPL_ISUPPORTS3(nsXvHandlerElement, nsIXvHandlerElement, nsIXTFGenericElement, nsIDOMEventListener)

#define ENTERFN( x )   printf( "****" #x " called.\n");

NS_HIDDEN_(nsresult)
NS_NewXvHandlerElement(nsIXTFElement **aResult)
{
  *aResult = new nsXvHandlerElement();
  if (!*aResult)
    return NS_ERROR_OUT_OF_MEMORY;

  NS_ADDREF(*aResult);
  return NS_OK;
}

nsXvHandlerElement::nsXvHandlerElement()
{
  printf("$$$$$$$$$$$$$nsXvHandlerElement::nsXvHandlerElement()\n");
  /* member initializers and constructor code */
}

nsXvHandlerElement::~nsXvHandlerElement()
{
  /* destructor code */
}

NS_IMETHODIMP nsXvHandlerElement::GetElementType(PRUint32 *aElementType)
{
  *aElementType = nsIXTFElement::ELEMENT_TYPE_GENERIC_ELEMENT;
  return NS_OK;
}

/* void GetScriptText (out wstring text); */
NS_IMETHODIMP nsXvHandlerElement::GetScriptText(PRUnichar **text)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

nsresult 
nsXvHandlerElement::SerializeContent(nsAString& aString)
{
  ENTERFN ( nsVxmlFormElement::SerializeContent )

  nsCOMPtr<nsIDOMSerializer> s = do_CreateInstance(NS_XMLSERIALIZER_CONTRACTID);
  NS_ASSERTION(s, "Failed to get nsIDOMSerializer.");

  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(mElement);
  NS_ASSERTION(node, "Failed to get nsIDOMNode.");

  nsCOMPtr<nsIDOMNode> child;
  node->GetFirstChild(getter_AddRefs(child));
  NS_ASSERTION(child, "Failed to get child node.");
 
  s->SerializeToString(child, aString);

  return NS_OK;
}

nsresult  
nsXvHandlerElement::InvokeForEvent(nsIDOMEvent *aEvent)
{
  //Get the script content
  nsAutoString script;
  SerializeContent(script);

  //Get the document so we can get the script global object
  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(mElement);
  NS_ASSERTION(node, "Element is not a node, we'll crash soon");

  nsCOMPtr<nsIDOMDocument> domdoc;
  node->GetOwnerDocument(getter_AddRefs(domdoc));
  nsCOMPtr<nsIDocument> doc;
  doc = do_QueryInterface(domdoc);

  //Get the context

	// Get the global object, context, and principles
	JSPrincipals *jsPrins = NULL;
	nsIScriptGlobalObject *sgo = doc->GetScriptGlobalObject();
	JSObject * glob = sgo->GetGlobalJSObject();
  nsIScriptContext *scrCx = sgo->GetContext();
  JSContext* cx = (JSContext *) scrCx->GetNativeContext();
	nsCOMPtr<nsIScriptObjectPrincipal> sop(do_QueryInterface(sgo));
	nsIPrincipal *prin = sop->GetPrincipal();
	prin->GetJSPrincipals(cx, &jsPrins);
	NS_ASSERTION(jsPrins, "Unable to get js principals!");

	// Evaluate the script
	// Push the context on the stack in FireFox
	JSBool jsok;
	jsval jsValue;
	nsresult rv;
	nsCOMPtr<nsIJSContextStack> cxstack = 
		do_GetService("@mozilla.org/js/xpc/ContextStack;1");
	rv = cxstack->Push(cx);
	if (NS_FAILED(rv)) {
		NS_ERROR("coudn't push the context on the stack");
		return NULL; //cc fix up just getting to compile here VXM_RC_INTERNAL_ERROR;
	}

	JS_BeginRequest(cx);

	// Evaluate the script
  int lineno=0;
	jsok = JS_EvaluateUCScriptForPrincipals(cx, glob, jsPrins, script.get(), script.Length(), "x+v script", lineno, &jsValue);

	JS_EndRequest(cx);

	JS_MaybeGC(cx);

	JSContext *cx2;
	cxstack->Pop(&cx2);
	NS_ASSERTION(cx == cx2, "JS contexts didn't match");

	if (!jsok)
	{
    //cc implement puting errors on the javascript console
    printf("nsXvHandlerElement::InvokeForEvent : Error evaluating JavaScript.");
  }

  return NS_OK;
}

//*********************************************************************************************************
// nsIDOMEventListener methods

NS_IMETHODIMP 
nsXvHandlerElement::HandleEvent(class nsIDOMEvent *aEvent)
{
  nsCOMPtr<nsIDOMEvent> event(aEvent);
  nsAutoString s;
  event->GetType(s);
  wprintf(L"nsXvHandlerElement::HandleEvent event type:%s\n", s.get());
  InvokeForEvent(aEvent);
  return NS_OK;
}


NS_IMETHODIMP nsXvHandlerElement::OnCreated(nsIXTFGenericElementWrapper *wrapper) 
{
  ENTERFN ( nsVxmlFormElement::OnCreated )

  wrapper->SetNotificationMask(nsIXTFElement::NOTIFY_DONE_ADDING_CHILDREN
                               |nsIXTFElement::NOTIFY_HANDLE_DEFAULT);

  nsCOMPtr<nsIDOMElement> element;
  wrapper->GetElementNode(getter_AddRefs(element));

  // It's ok to keep a weak pointer to mElement.  mElement will have an
  // owning reference to this object, so as long as we null out mElement in
  // OnDestroyed, it will always be valid.

  mElement = element;
  NS_ASSERTION(mElement, "Wrapper is not an nsIDOMElement, we'll crash soon");

  return NS_OK;
}

NS_IMETHODIMP 
nsXvHandlerElement::OnDestroyed(void) 
{
  // release our node
  mElement = nsnull;
  return NS_OK;
}

//NS_IMETHODIMP nsXvHandlerElement::GetElementType(PRUint32 *aElementType);
NS_IMETHODIMP nsXvHandlerElement::GetIsAttributeHandler(PRBool *aIsAttributeHandler) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::GetScriptingInterfaces(PRUint32 *count, nsIID * **array) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillChangeDocument(nsIDOMDocument *newDoc) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::DocumentChanged(nsIDOMDocument *newDoc) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillChangeParent(nsIDOMElement *newParent) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::ParentChanged(nsIDOMElement *newParent) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillInsertChild(nsIDOMNode *child, PRUint32 index) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::ChildInserted(nsIDOMNode *child, PRUint32 index) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillAppendChild(nsIDOMNode *child) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::ChildAppended(nsIDOMNode *child) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillRemoveChild(PRUint32 index) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::ChildRemoved(PRUint32 index) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillSetAttribute(nsIAtom *name, const nsAString & newValue) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::AttributeSet(nsIAtom *name, const nsAString & newValue) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::WillRemoveAttribute(nsIAtom *name) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::AttributeRemoved(nsIAtom *name) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::BeginAddingChildren(void) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::DoneAddingChildren(void) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::HandleDefault(nsIDOMEvent *aEvent, PRBool *_retval) {return NS_OK;}
NS_IMETHODIMP nsXvHandlerElement::CloneState(nsIDOMElement *aElement) {return NS_OK;}

